import { Observable } from "rxjs/Observable";

export function createObservableThunk(options) {
  var method = options.method,
      before = options.before,
      success = options.success,
      failure = options.failure;


  var beforeActions = ensureArray(before);
  var successActions = ensureArray(success);
  var failureActions = ensureArray(failure);

  return function thunkCreator(input) {
    return function thunk(dispatch, getState) {
      dispatchMany(beforeActions, input);

      var o = method(input).do(function (response) {
        return dispatchMany(successActions, response);
      }).catch(function (error) {
        return dispatchMany(failureActions, error) || Observable.of({ error: error });
      }).share();

      o.subscribe();

      return o;

      function dispatchMany(actionCreators, input) {
        var date = new Date();
        actionCreators.forEach(function (callback) {
          var action = callback(input, date);
          if (action) {
            dispatch(action);
          } else {
            console.error("Invalid Action Creator " + callback.name + " returned undefined");
          }
        });
      }
    };
  };
}

function ensureArray(callbacks) {
  if (Array.isArray(callbacks)) {
    return callbacks;
  }

  if (callbacks) {
    return [callbacks];
  }

  return [];
}